@@ -2,56 +2,30 @@
 # pylint: disable=line-too-long
 
 '''
-    Module for basic image manipulations.
+    A module of utility methods used for manipulating image files.
+
+    ----------
+
+    Meta
+    ----------
+    `author`: Colemen Atwood
+    `created`: 06-04-2022 10:58:25
+    `memberOf`: file_utils
 '''
 
-# import subprocess
-# import json
-# import shutil
-# import os
-# import ffmpeg
-# from distutils import extension
-
-import time
-import os
+import time as _time
+import os as _os
+import re as _re
+from typing import Union as _Union
+from PIL import Image as _Image
+import utils.file_utils as _f
+import utils.dict_utils as _obj
+import utils.string_utils as _csu
+import utils.directory_utils as _dir
 
 
-# import json
-import re
-from PIL import Image
-# import re
-# from pathlib import Path
-# import utils.files.file as f
-# import utils.object_utils as obj
-# import utils.file_read as read
-# import colemen_string_utils as csu
-# import facades.string_facade as csu
-# import colemen_string_utils as strUtils
-# import utils.files.exiftool as exiftool
+REGEX_TYPE = type(_re.compile('hello, world'))
 
-import utils.file_utils as f
-import utils.dict_utils as obj
-import utils.string_utils as csu
-
-# from utils.dict_utils.dict_utils import get_kwarg as obj.get_kwarg
-
-# from utils.file_utils.file_utils import get_data as f.get_data
-# from utils.file_utils.file_utils import exists as f.exists
-# from utils.file_utils.file_utils import delete as f.delete
-# from utils.file_utils.file_image import get_keywords as f.get_keywords
-# from utils.file_utils.file_image import save_file_obj as f.save_file_obj
-# from utils.file_utils.file_search import by_name as f.by_name
-
-# from utils.string_utils.string_format import to_snake_case as csu.to_snake_case
-# from utils.string_utils.string_format import file_path as csu.file_path
-# from utils.string_utils.string_generation import to_hash as csu.to_hash
-
-
-
-
-REGEX_TYPE = type(re.compile('hello, world'))
-
-# from threading import Thread
 
 def _parse_file_obj_list(file):
     '''
@@ -80,14 +54,14 @@ def _parse_file_obj_list(file):
     '''
     file_array = []
     if isinstance(file, (str)):
-        if f.exists(file):
-            file_array.append(f.get_data(file))
+        if _f.exists(file):
+            file_array.append(_f.get_data(file))
 
     if isinstance(file, (list)):
         for i in file:
             if isinstance(i, (str)):
-                if f.exists(i):
-                    file_array.append(f.get_data(i))
+                if _f.exists(i):
+                    file_array.append(_f.get_data(i))
             if isinstance(i, (dict)):
                 if 'file_path' in i:
                     file_array.append(i)
@@ -100,7 +74,7 @@ def _parse_file_obj_list(file):
 
 def _keywords_to_list(keywords,delimiter=",",**kwargs):
     # print(f"")
-    to_snake_case = obj.get_kwarg(['snake case'],True,(bool),**kwargs)
+    to_snake_case = _obj.get_kwarg(['snake case'],True,(bool),**kwargs)
     new_keys = []
     # print(f"_keywords_to_list: {keywords} {type(keywords)}")
 
@@ -129,7 +103,7 @@ def _keywords_to_list(keywords,delimiter=",",**kwargs):
     # print(f"newlist: ",new_keys)
     newlist = list(set(new_keys))
     if to_snake_case is True:
-        newlist = [csu.to_snake_case(x) for x in newlist]
+        newlist = [_csu.to_snake_case(x) for x in newlist]
 
     return newlist
 
@@ -138,34 +112,86 @@ def gen_css_media_scales(src_path):
     for size in sizes:
         scale(src_path, (size, size), keep_proportion=True)
 
-def scale(src_path,size,**kwargs):
+def scale(src_path:str,size:_Union[list,tuple],**kwargs):
+    '''
+        Scale an image to a new size.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `src_path` {str}
+            The path to the image to scale.
+
+        `size` {list|tuple}
+            A lsit or tuple [width,height]
+
+        `keep_proportion` {bool}
+            if False, it will not attempt to keep the images proportions.
+
+        Return {None}
+        ----------------------
+        returns nothing.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-04-2022 13:11:55
+        `memberOf`: file_image
+        `version`: 1.0
+        `method_name`: scale
+        * @xxx [06-04-2022 13:14:03]: documentation for scale
+    '''
+
+
     if isinstance(size,(list,tuple)):
         width = size[0]
         height = size[1]
     else:
-        print("size must be a list or tuple [width,height]")
+        print("Size must be a list or tuple [width,height]")
         return False
 
 
-    dst_path = obj.get_kwarg(['dst_path'], False, (str), **kwargs)
-    keep_proportion = obj.get_kwarg(['keep_proportion'], True, (bool), **kwargs)
+    dst_path = _obj.get_kwarg(['dst_path'], False, (str), **kwargs)
+    keep_proportion = _obj.get_kwarg(['keep_proportion'], True, (bool), **kwargs)
 
-    sizeTuple = (width,height)
+    size_tuple = (width,height)
     if keep_proportion is True:
         if width > height:
-            sizeTuple = (width,width)
+            size_tuple = (width,width)
         else:
-            sizeTuple = (height, height)
+            size_tuple = (height, height)
 
-    file_data = f.get_data(src_path)
+    file_data = _f.get_data(src_path)
     if dst_path is False:
-        dst_path = f"{file_data['dir_path']}/{file_data['name_no_ext']}_{sizeTuple[0]}x{sizeTuple[1]}{file_data['extension']}"
+        dst_path = f"{file_data['dir_path']}/{file_data['name_no_ext']}_{size_tuple[0]}x{size_tuple[1]}{file_data['extension']}"
 
-    image = Image.open(src_path)
-    image.thumbnail(sizeTuple, Image.ANTIALIAS)
+    image = _Image.open(src_path)
+    image.thumbnail(size_tuple, _Image.ANTIALIAS)
     image.save(dst_path)
 
-def delete_all_keywords(files):
+def delete_all_keywords(files:list)->None:
+    '''
+        Remove all keywords from the files
+
+        ----------
+
+        Arguments
+        -------------------------
+        `files` {list}
+            A list of file paths or file data dictionary's
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-04-2022 13:16:09
+        `memberOf`: file_image
+        `version`: 1.0
+        `method_name`: delete_all_keywords
+        * @xxx [06-04-2022 13:18:02]: documentation for delete_all_keywords
+    '''
+
+
     file_array = _parse_file_obj_list(files)
     for file in file_array:
         if 'meta_data' not in file:
@@ -174,8 +200,8 @@ def delete_all_keywords(files):
         purge_original(file)
 
 def delete_keyword(files,keywords='*',**kwargs):
-    case_sensitive = obj.get_kwarg(['case sensitive'],True,(bool),**kwargs)
-    save = obj.get_kwarg(['save'],False,(bool),**kwargs)
+    case_sensitive = _obj.get_kwarg(['case sensitive'],True,(bool),**kwargs)
+    save = _obj.get_kwarg(['save'],False,(bool),**kwargs)
     needle_array = _keywords_to_list(keywords)
     update_array = []
     if len(needle_array) == 0:
@@ -206,7 +232,7 @@ def delete_keyword(files,keywords='*',**kwargs):
                             new_keys.append(haystack)
                         # else:
                         #     matchFound = True
-        # f.write.to_json("imgs.json",file)
+        # _f.write.to_json("imgs.json",file)
         # exit()
         if len(new_keys) > 1:
             new_keys = list(set(new_keys))
@@ -222,8 +248,8 @@ def delete_keyword(files,keywords='*',**kwargs):
             # purge_original(file)
 
 def add_keyword(files,keywords='',**kwargs):
-    snake_case = obj.get_kwarg(['snake case'],True,(bool),**kwargs)
-    save = obj.get_kwarg(['save'],False,(bool),**kwargs)
+    snake_case = _obj.get_kwarg(['snake case'],True,(bool),**kwargs)
+    save = _obj.get_kwarg(['save'],False,(bool),**kwargs)
 
     file_array = _parse_file_obj_list(files)
     # print(f"add_keyword.keywords: ",keywords)
@@ -279,8 +305,8 @@ def set_keywords(files,keywords='',**kwargs):
         `version`: 1.0
         `method_name`: set_keywords
     '''
-    save = obj.get_kwarg(['save'],False,(bool),**kwargs)
-    snake_case = obj.get_kwarg(['snake case'],True,(bool),**kwargs)
+    save = _obj.get_kwarg(['save'],False,(bool),**kwargs)
+    snake_case = _obj.get_kwarg(['snake case'],True,(bool),**kwargs)
 
     file_array = _parse_file_obj_list(files)
     keywords = _keywords_to_list(keywords,",",snake_case=snake_case)
@@ -301,10 +327,10 @@ def set_keywords(files,keywords='',**kwargs):
 def save_file_obj(files):
     file_array = _parse_file_obj_list(files)
 
-    with f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
+    with _f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
         for file in file_array:
             if 'meta_data' in file:
-                print(f"save_file_obj.filePath: {file['file_path']}")
+                print(f"save_file__obj.filePath: {file['file_path']}")
                 et.set_tags(file['meta_data'],file['file_path'])
 
 def tags_to_snakecase(files):
@@ -314,7 +340,7 @@ def tags_to_snakecase(files):
         if 'meta_data' not in file:
             print("file missing meta_data")
             file = get_meta(file)
-        # f.write.to_json("result_array.json",file)
+        # _f.write.to_json("result_array.json",file)
 
         # xmpTags = file['meta_data']['XMP:Subject']
         # iptcTags = file['meta_data']['IPTC:Keywords']
@@ -327,7 +353,7 @@ def tags_to_snakecase(files):
         # print(f"tags: ",tags)
         new_tags = []
         for tag in tags:
-            new_tags.append(csu.to_snake_case(tag))
+            new_tags.append(_csu.to_snake_case(tag))
         if len(new_tags) > 1:
             new_tags = list(set(new_tags))
         file['meta_data']['XMP:Subject'] = new_tags
@@ -344,11 +370,11 @@ def get_meta_only(file_path):
 
     # paths_array = []
     # for path in file_path:
-    #     if f.exists(path):
+    #     if _f.exists(path):
     #         paths_array.append(path)
 
-    # et = f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe")
-    with f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
+    # et = _f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe")
+    with _f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
         meta_data = et.get_metadata_batch(file_path)[0]
         if 'XMP:Subject' not in meta_data:
             meta_data['XMP:Subject'] = []
@@ -359,14 +385,14 @@ def get_meta_only(file_path):
 def get_meta(files,force_update=False):
     result_array = []
     file_array = _parse_file_obj_list(files)
-    # et = f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe")
+    # et = _f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe")
     for file in file_array:
-        file['file_path'] = csu.file_path(file['file_path'],url=True)
-        file['file_path_exif_copy'] = csu.file_path(f"{file['file_path']}_original",url=True)
+        file['file_path'] = _csu.file_path(file['file_path'],url=True)
+        file['file_path_exif_copy'] = _csu.file_path(f"{file['file_path']}_original",url=True)
 
         if 'meta_data' not in file or force_update is True:
         # print(f"get_meta.file_path: {file['file_path']}")
-            with f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
+            with _f.exiftool.ExifTool(executable_=r"Z:\Structure\Archive\Programming Packages - Libraries\Python\colemen_file_utils\.venv\exiftool.exe") as et:
                 file['meta_data'] = et.get_metadata_batch([file['file_path']])[0]
                 if 'XMP:Subject' not in file['meta_data']:
                     file['meta_data']['XMP:Subject'] = []
@@ -374,9 +400,9 @@ def get_meta(files,force_update=False):
                     file['meta_data']['IPTC:Keywords'] = []
         result_array.append(file)
 
-    #     # file = f.get_data(file_path)
+    #     # file = _f.get_data(file_path)
     #     file['update_file'] = False
-    #     # im = Image.open(x['file_path'])
+    #     # im = _Image.open(x['file_path'])
     #     info = IPTCInfo(file['file_path'], force=True)
     #     # print(info.__dict__.items())
     #     for k, v in info.__dict__.items():
@@ -396,8 +422,8 @@ def purge_original(files):
     for file in file_array:
         if 'meta_data' not in file:
             file = get_meta(file)
-        if f.exists(file['file_path_exif_copy']):
-            f.delete(file['file_path_exif_copy'])
+        if _f.exists(file['file_path_exif_copy']):
+            _f.delete(file['file_path_exif_copy'])
 
 def get_keywords(files):
     # print(f"")
@@ -488,7 +514,7 @@ def replace_keyword(files,needle,replace):
 
 def has_keyword(files,keywords,**kwargs):
     # Causes the search to reverse, so if the image does NOT have a tag, it is returned
-    reverse = obj.get_kwarg(['reverse'],False,(bool),**kwargs)
+    reverse = _obj.get_kwarg(['reverse'],False,(bool),**kwargs)
     keyword_array = _keywords_to_list(keywords,",",snake_case=False)
     file_array = _parse_file_obj_list(files)
     result_array = []
@@ -554,7 +580,7 @@ def apply_keyword_synonyms(files,synonyms,**kwargs):
         `version`: 1.0
         `method_name`: apply_keyword_synonyms
     '''
-    save = obj.get_kwarg(['save'],True,(bool),**kwargs)
+    save = _obj.get_kwarg(['save'],True,(bool),**kwargs)
     # Prepare the files array for iteration and gather necessary info.
     files_array = _parse_file_obj_list(files)
     update_array = []
@@ -566,8 +592,8 @@ def apply_keyword_synonyms(files,synonyms,**kwargs):
         return files_array
 
     for file in files_array:
-        kws = f.get_keywords(file)
-        kws_hash_prime = csu.to_hash(kws)
+        kws = _f.get_keywords(file)
+        kws_hash_prime = _csu.to_hash(kws)
         if isinstance(kws,(list)) is False:
             kws = [kws]
         # new_tags = kws
@@ -586,7 +612,7 @@ def apply_keyword_synonyms(files,synonyms,**kwargs):
 
         # print(f"kws_hash_prime: {kws_hash_prime}")
         # kws = cfu.file.image.get_keywords(file)
-        kwsHash = csu.to_hash(new_tags)
+        kwsHash = _csu.to_hash(new_tags)
         # print(f"kwsHash:        {kwsHash}")
 
         # compare the original keyword hash to the new one
@@ -602,13 +628,13 @@ def apply_keyword_synonyms(files,synonyms,**kwargs):
 
     # if any files need updating, then we save the data to the file..
     if len(update_array) > 0 and save is True:
-        f.save_file_obj(update_array)
+        _f.save_file_obj(update_array)
     return files_array
 
 def apply_synonyms(file_keys,synonyms):
     if len(file_keys) > 1:
         file_keys = list(set(file_keys))
-    # file_keys_hash = csu.to_hash(file_keys)
+    # file_keys_hash = _csu.to_hash(file_keys)
     added_keys = False
     new_file_keys = file_keys
     for k,v in synonyms.items():
@@ -625,7 +651,7 @@ def apply_synonyms(file_keys,synonyms):
     if len(new_file_keys) > 1:
         new_file_keys = list(set(new_file_keys))
 
-    # new_file_keys_hash = csu.to_hash(new_file_keys)
+    # new_file_keys_hash = _csu.to_hash(new_file_keys)
     # if new_file_keys_hash != file_keys_hash:
     if added_keys is True:
         return apply_synonyms(new_file_keys,synonyms)
@@ -657,10 +683,10 @@ def auto_tag_by_folder_name(watch_path,**kwargs):
         `method_name`: auto_tag_by_folder_name
     '''
     print("auto_tag_by_folder_name")
-    synonyms = obj.get_kwarg(['synonyms'],False,(bool,dict),**kwargs)
-    apply_current_tags = obj.get_kwarg(['apply_current_tags'],False,(bool),**kwargs)
+    synonyms = _obj.get_kwarg(['synonyms'],False,(bool,dict),**kwargs)
+    apply_current_tags = _obj.get_kwarg(['apply_current_tags'],False,(bool),**kwargs)
     print(f"Indexing images in {watch_path}")
-    all_files = f.get_files(watch_path, ext="images",include_meta_data=True)
+    all_files = _f.get_files(watch_path, ext="images",include_meta_data=True)
     print(f"Watching {len(all_files)} files")
     # duration = 0
 
@@ -668,9 +694,9 @@ def auto_tag_by_folder_name(watch_path,**kwargs):
         apply_current_folders_as_tag(all_files)
 
     while True:
-        time.sleep(.5)
+        _time.sleep(.5)
         if len(all_files) == 0:
-            all_files = f.get_files(watch_path, ext="images",include_meta_data=True)
+            all_files = _f.get_files(watch_path, ext="images",include_meta_data=True)
         # duration += .5
         # if duration % 30 == 0:
         #     transferCompleteImages.transferCompleteImages()
@@ -682,7 +708,7 @@ def auto_tag_by_folder_name(watch_path,**kwargs):
 
         for file in all_files:
             # if the file does not exist, append it to the moved_files list.
-            if f.exists(file['file_path']) is False:
+            if _f.exists(file['file_path']) is False:
                 print(f"moved: {file['file_name']}")
                 moved_files.append(file)
             else:
@@ -690,15 +716,15 @@ def auto_tag_by_folder_name(watch_path,**kwargs):
 
         for file in moved_files:
             # search for the file by its name
-            result = f.by_name(file['file_name'], watch_path)
+            result = _f.by_name(file['file_name'], watch_path)
             if result is not None:
                 # print(f"result: {result}")
                 # get the file data, essentially update its paths and such.
-                file_data = f.get_data(result[0])
+                file_data = _f.get_data(result[0])
                 # append the file to the update_files array
                 update_files.append(file_data)
 
-        new_files = _tag_by_folder_name(update_files,os.path.basename(watch_path),save=False)
+        new_files = _tag_by_folder_name(update_files,_os.path.basename(watch_path),save=False)
         if synonyms is not False:
             new_files = apply_keyword_synonyms(new_files,synonyms,save=False)
         save_file_obj(new_files)
@@ -707,16 +733,16 @@ def auto_tag_by_folder_name(watch_path,**kwargs):
         all_files = new_all_files
 
 def apply_current_folders_as_tag(files,**kwargs):
-    save = obj.get_kwarg(['save'],False,(bool),**kwargs)
-    if f.directory.exists(files):
-        files = f.get_files(files, ext="images",include_meta_data=True)
+    save = _obj.get_kwarg(['save'],False,(bool),**kwargs)
+    if _dir.exists(files):
+        files = _f.get_files(files, ext="images",include_meta_data=True)
     # if files is False:
     files_array = _parse_file_obj_list(files)
     update_array = []
     # new_file_array = []
     for file in files_array:
         # parse the folder name from the file object
-        dir_name = os.path.basename(file['dir_path'])
+        dir_name = _os.path.basename(file['dir_path'])
         file = add_keyword(file,dir_name,save=False)[0]
         update_array.append(file)
     if save is True:
@@ -724,7 +750,7 @@ def apply_current_folders_as_tag(files,**kwargs):
     return files_array
 
 def _tag_by_folder_name(files,exclude='',**kwargs):
-    save = obj.get_kwarg(['save'],False,(bool),**kwargs)
+    save = _obj.get_kwarg(['save'],False,(bool),**kwargs)
     if isinstance(exclude,(str)):
         exclude = [exclude]
     files_array = _parse_file_obj_list(files)
@@ -732,7 +758,7 @@ def _tag_by_folder_name(files,exclude='',**kwargs):
     # new_file_array = []
     for file in files_array:
         # parse the folder name from the file object
-        dir_name = os.path.basename(file['dir_path'])
+        dir_name = _os.path.basename(file['dir_path'])
         if dir_name not in exclude:
             file = add_keyword(file,dir_name,save=False)[0]
             update_array.append(file)
@@ -795,7 +821,7 @@ def tag_if_missing_keyword(files,needles,keyword):
 #             # info['supplemental category'] = file['iptc_data']['supplemental category']
 #             # info['Contact'] = file['iptc_data']['Contact']
 #             info.save_as(file['file_path'])
-#             f.delete(f'{file["file_path"]}~')
+#             _f.delete(f'{file["file_path"]}~')
 
 # def formatIPTCData(d):
 #     data = {}
